<?php

class Zenmall_Data{

  public $shopInfo = array();
  public $currencies = array();
  public $currenciesId = array();
  public $categories = array();
  public $offers = array();
  public $allSectionsId = array();

  public function __construct(nc_netshop $netshop, $domain){
    $this->netshop = $netshop;
    $this->nc_core = nc_core();
    $this->domain = !empty($domain) ? $domain : $this->nc_core->DOMAIN_NAME;
    $this->getShopInfo();
    $this->getCurrencies();
    $this->getCategories();
    $this->getOffers();
  }

  public function getShopInfo(){
    $this->shopInfo["name"] = $this->netshop->get_setting('ShopName');
    $this->shopInfo["company"] = $this->netshop->get_setting('CompanyName');
    $this->shopInfo["url"] = "http://" . $this->domain . "/";
    $this->shopInfo["platform"] = "NetCat";
    $this->shopInfo["email"] = $this->netshop->get_setting('ManagerEmail');

  }

  public function getCurrencies(){
    $currencies = $this->netshop->get_setting('Currencies');
    if ($this->netshop->get_setting('DefaultCurrencyID') > 0) {
      $default_currency = $currencies[$this->netshop->get_setting('DefaultCurrencyID')];
    } else {
      $default_currency = reset($currencies);
    }
    $rates = $this->netshop->get_setting('Rates');

    foreach ($currencies as $key => $currency) {
      $this->currenciesId[$key] = $currency;
      if ($currency != $default_currency) {
        $this->currencies[] = array(
          "id"=>$currency, 
          "rate"=>(isset($rates[$key]) && $rates[$key] > 0) ? $rates[$key] : "CBRF",
        );
      }else{
        $this->currencies[] = array(
          "id"=>$currency, 
          "rate"=>1,
        );
      }
    }
  }

  public function getCategories(){
    $structure = GetStructureYandexml(implode(",", $this->netshop->get_goods_components_ids()), $this->netshop->get_catalogue_id());
    
    if (is_array($structure) && count($structure) > 0) {
      foreach ($structure as $category) {
        $this->categories[] = array(
          'id'=>$category['Subdivision_ID'],
          'name'=>$category["Subdivision_Name"],
          'parent_id'=>array_key_exists($category['Parent_Sub_ID'], $structure) ? $category['Parent_Sub_ID'] : null,
        );
        $this->allSectionsId[$category["Class_ID"]] = $category["Subdivision_ID"];
      }
    }
  }

  public function getOffers(){
    $db = nc_db();
    foreach ($this->netshop->get_goods_components_ids() as $goods_table) {
      $sql = "SELECT `Field_ID`, `Field_Name`, `Description` FROM `Field` WHERE `Class_ID` = {$goods_table}";
      $netcat_fields = array();

      $parent_fields = "";
      $parent_join = "";

      foreach ((array) $db->get_results($sql, ARRAY_A) as $netcat_field) {
        $netcat_fields[$netcat_field['Field_ID']] = $netcat_field['Field_Name'];
        if ($netcat_field['Field_Name'] == 'Name') {
          $parent_fields .= ", parent.Name as Parent_Name, parent.Description as Parent_Description, parent.Image as Parent_Image, parent.BigImage as Parent_BigImage ";
          $parent_join = " LEFT JOIN `Message{$goods_table}` as parent ON parent.Message_ID = m.Parent_Message_ID AND m.Parent_Message_ID <> 0 ";
        }

        $goods_data = (array) $db->get_results("SELECT m.*, CONCAT(u.Hidden_URL, s.EnglishName, '_', m.Message_ID, '.html') as URL"
          . $parent_fields
          . " FROM `Message{$goods_table}` as m "
          . " JOIN `Subdivision` as u ON m.`Subdivision_ID`=u.`Subdivision_ID` "
          . " JOIN `Sub_Class` as s ON s.`Subdivision_ID` = m.`Subdivision_ID` AND s.`Sub_Class_ID` = m.`Sub_Class_ID` "
          . $parent_join
          . "WHERE m.`Checked` = 1 AND m.`Subdivision_ID` IN (" . $this->allSectionsId[$goods_table] . ")"
          . "", ARRAY_A);

        foreach ($goods_data as $row) {
          $offer = array('data'=>array(), 'params'=>array(), 'id'=>sprintf("%d%05d", $goods_table, $row["Message_ID"]));
          // stock hook
          if (isset($row["StockUnits"]) && strlen($row["StockUnits"])) {
            $offer["available"] = ($row["StockUnits"] ? "true" : "false");
          } else {
            $offer["available"] = "true";
          }
          // image hook
          if (isset($row["Image"]) && $row["Image"] != "") {
            $offer['data']["picture"] = "http://" . $this->domain . nc_file_path($goods_table, $row["Message_ID"], "Image", "h_");
          }elseif(isset($row["Parent_Image"]) && $row["Parent_Image"] != ""){
            $offer['data']["picture"] = "http://" . $this->domain . nc_file_path($goods_table, $row["Parent_Message_ID"], "Image", "h_");
          }
          // name hook
          if (isset($row["Name"]) && $row["Name"] != "") {
            $offer['data']["name"] = $row["Name"];
          }elseif(isset($row["Parent_Name"]) && $row["Parent_Name"] != ""){
            $offer['data']["name"] = $row["Parent_Name"];
          }
          if (isset($row["Price"]) && $row["Price"] != "") {
            $offer['data']["price"] = $row["Price"];
          }
          if (isset($row["Description"]) && strip_tags($row["Description"]) != "") {
            $offer['data']["description"] = strip_tags($row["Description"]);
          }elseif (isset($row["Parent_Description"]) && strip_tags($row["Parent_Description"]) != "") {
            $offer['data']["description"] = strip_tags($row["Parent_Description"]);
          }
          if (isset($row["Color"]) && $row["Color"] != "") {
            $offer['params'][] = array('name'=>"Цвет", 'text'=>$row["Color"]);
          }
          $offer['data']['url'] = "http://" . $this->domain . $row["URL"];
          $offer['data']["currencyId"] = $this->currenciesId[$row["Currency"]];
          $offer['data']["categoryId"] = $this->allSectionsId[$goods_table];
          
          // var_dump($row);
          $this->offers[$offer['id']] = $offer;
        }
      }
    }
  }
}


class Zenmall_XMLGen {

  /**
     * Xml file encoding
     * @var string
     */
    public $encoding = 'windows-1251';
    
    /**
     * Output file name. If null 'php://output' is used.
     * @var string
     */
    public $outputFile;
    
    /**
     * Indent string in xml file. False or null means no indent;
     * @var string
     */
    public $indentString = "\t";
    
    /**
     * An array of element names used to describe your web-shop according to YML standart
     * @var array
     */
    public $shopInfoElements = array('name','company','url','platform','version','agency','email');
    
    /**
     * An array of element names used to create an offer according to YML standart
     * @var array
     */
    public $offerElements = array('url', 'price', 'currencyId', 'categoryId', 'market_category', 
            'picture', 'store', 'pickup', 'delivery', 'local_delivery_cost','typePrefix', 
            'vendor', 'vendorCode', 'name' ,'model', 'description', 'sales_notes', 'manufacturer_warranty',
            'seller_warranty','country_of_origin', 'downloadable', 'age','barcode','cpa',
            'rec','expiry','weight','dimensions','param');
    
    protected $_dir;
    protected $_file;
    protected $_tmpFile;
    protected $_engine;

    public $shopInfo = array(
        'name'=>'',
        'company'=>'',
        'url'=>'',
        'platform'=>'',
        'email'=>''
    );

    public $currencies = array();
    public $categories = array();
    public $offers = array();
    
    public function run() {
        $this->beforeWrite();
        
        $this->writeShopInfo();
        $this->writeCurrencies();
        $this->writeCategories();
        $this->writeOffers();
        
        $this->afterWrite();
    }
    
    protected function beforeWrite() {
        if ($this->outputFile !== null) {
            $slashPos = strrpos($this->outputFile, DIRECTORY_SEPARATOR);
            if (false !== $slashPos) {
                $this->_file = substr($this->outputFile, $slashPos);
                $this->_dir = substr($this->outputFile, 0, $slashPos);
            }
            else {
               $this->_dir = ".";
            }
            $this->_tmpFile = $this->_dir.DIRECTORY_SEPARATOR.md5($this->_file);
        }
        else {
            $this->_tmpFile = 'php://output';
        }
        $engine = $this->getEngine();
        $engine->openURI($this->_tmpFile);
        if ($this->indentString) {
            $engine->setIndentString($this->indentString);
            $engine->setIndent(true);
        }
        $engine->startDocument('1.0',$this->encoding);
        $engine->startElement('yml_catalog');
        $engine->writeAttribute('date', date('Y-m-d H:i'));
        $engine->startElement('shop');
    }
    
    protected function afterWrite() {
        $engine = $this->getEngine();
        $engine->fullEndElement();
        $engine->fullEndElement();
        $engine->endDocument(); 
        
        if (null !== $this->outputFile)
            rename($this->_tmpFile, $this->outputFile);
    }
    
    protected function getEngine() {
        if (null === $this->_engine) {
            $this->_engine = new XMLWriter();
        }
        return $this->_engine;
    }
    protected function writeShopInfo() {
        $engine = $this->getEngine();
        foreach($this->shopInfo() as $elm=>$text) {
            if (in_array($elm,$this->shopInfoElements)) {
                $engine->writeElement($elm, $text);
            }
        }
    }
    
    protected function writeCurrencies() {
        $engine = $this->getEngine();
        $engine->startElement('currencies');
        $this->currencies();
        $engine->fullEndElement();
    }
    
    protected function writeCategories() {
        $engine = $this->getEngine();
        $engine->startElement('categories');
        $this->categories();
        $engine->fullEndElement();
    }
    
    protected function writeOffers() {
        $engine = $this->getEngine();
        $engine->startElement('offers');
        $this->offers();
        $engine->fullEndElement();
    }
    
    /**
     * Adds <currency> element. (See http://help.yandex.ru/partnermarket/currencies.xml)
     * @param string $id "id" attribute 
     * @param mixed $rate "rate" attribute
     */
    protected function addCurrency($id,$rate = 1) {
        $engine = $this->getEngine();
        $engine->startElement('currency');
        $engine->writeAttribute('id', $id);
        $engine->writeAttribute('rate', $rate);
        $engine->endElement();
    }
    
    /**
     * Adds <category> element. (See http://help.yandex.ru/partnermarket/categories.xml)
     * @param string $name category name
     * @param int $id "id" attribute
     * @param int $parentId "parentId" attribute
     */
    protected function addCategory($name,$id,$parentId = null) {
        $engine = $this->getEngine();
        $engine->startElement('category');
        $engine->writeAttribute('id', $id);
        if ($parentId)
            $engine->writeAttribute('parentId', $parentId);
        $engine->text($name);
        $engine->fullEndElement();
    }
    
    /**
     * Adds <offer> element. (See http://help.yandex.ru/partnermarket/offers.xml)
     * @param int $id "id" attribute
     * @param array $data array of subelements as elementName=>value pairs
     * @param array $params array of <param> elements. Every element is an array: array(NAME,UNIT,VALUE) (See http://help.yandex.ru/partnermarket/param.xml)
     * @param boolean $available "available" attribute
     * @param string $type "type" attribute
     * @param numeric $bid "bid" attribute
     * @param numeric $cbid "cbid" attribute
     */
    protected function addOffer($id,$data, $params = array(), $available=true, $type = 'vendor.model', $bid = null, $cbid = null) {
        $engine = $this->getEngine();
        $engine->startElement('offer');
        $engine->writeAttribute('id', $id);
        if ($type) 
            $engine->writeAttribute('type', $type);
        $engine->writeAttribute('available', $available ? 'true' : 'false');
        if ($bid) {
            $engine->writeAttribute('bid', $bid);
            if ($cbid) 
                $engine->writeAttribute('cbid', $cbid);
        }
        foreach($data as $elm=>$val) {
            if (in_array($elm,$this->offerElements)) {
                if (!is_array($val)) {
                    $val = array($val);
                }
                foreach($val as $value) {
                    $engine->writeElement($elm, $value);
                }
            }
        }
        if(count($params)>0){
          foreach($params as $param) {
               $engine->startElement('param');
               $engine->writeAttribute('name', $param['name']);
               if (isset($param['unit']))
                   $engine->writeAttribute('unit', $param['unit']);
               $engine->text($param['text']);
               $engine->endElement();
          }
        }
        $engine->fullEndElement();
    }
    
    protected function shopInfo(){
        return $this->shopInfo;
    }
    
    protected function currencies(){
        foreach($this->currencies as $currecy) {
            $this->addCurrency($currecy['id'],$currecy['rate']);
        }
    }
    
    protected function categories(){
        foreach($this->categories as $category) {
            $this->addCategory($category['name'],$category['id'],$category['parent_id']);
        } 
    }
    
    protected function offers(){
        foreach($this->offers as $offer) {
            $this->addOffer($offer['id'],$offer['data'], $offer['params'], $offer['available'], $offer['type'], $offer['bid'], $offer['cbid']);
        }
    }
}